/*******************************************************************************
 *******************************************************************************
 */

// Arduino environment
#if ARDUINO >= 100
#include <Arduino.h>
#else
#include <WProgram.h>
#endif


#include "SoftTimer.h"


bool SoftTimer::is_expired()
{
  return millis() >= _finish_ms;
}

void SoftTimer::start(uint16_t delay_ms)
{
  _finish_ms = millis() + delay_ms;
}

/*
 * Use unsigned long for argument, because that is what millis() returns.
 */
void SoftTimer::startW(unsigned long delay_ms)
{
  _finish_ms = millis() + delay_ms;
}

void SoftTimer::startN(uint8_t delay_ms)
{
  _finish_ms = millis() + delay_ms;
}

bool SoftTimer::interval(uint16_t delay_ms)
{
  bool exp = is_expired();
  if (exp)
    start(delay_ms);
  return exp;
}

/*
 * Use unsigned long for argument, because that is what millis() returns.
 */
bool SoftTimer::intervalW(unsigned long delay_ms)
{
  bool exp = is_expired();
  if (exp)
    startW(delay_ms);
  return exp;
}

bool SoftTimer::intervalN(uint8_t delay_ms)
{
  bool exp = is_expired();
  if (exp)
    startN(delay_ms);
  return exp;
}


SoftTimerExt::SoftTimerExt()
{
}

SoftTimerExt::SoftTimerExt(unsigned long delay_ms)
{
  startW(delay_ms);
}

void SoftTimerExt::restart()
{
  startW(_length_ms);
}

uint32_t SoftTimerExt::remaining()
{
  return _finish_ms - millis();
}

/*
 * This returns ceil() of the remaing seconds.
 * I.e. 0s returns 0, 0.001 .. 1.000s returns 1, 1.001s .. 2.000s returns 2.
 */
uint32_t SoftTimerExt::remaining_s()
{
  return (remaining() + 999) / 1000;
}

uint32_t SoftTimerExt::elapsed()
{
  return millis() - (_finish_ms - _length_ms);
}

uint32_t SoftTimerExt::elapsed_s()
{
  return elapsed() / 1000;
}

bool SoftTimerExt::is_new_s()
{
  uint8_t this_s = remaining_s();
  if (this_s != _last_s) {
    _last_s = this_s;
    return true;
  }
  else
    return false;
}

void SoftTimerExt::start(uint16_t delay_ms)
{
  // Start timer before saving delay value.
  SoftTimer::start(delay_ms);
  _length_ms = delay_ms;
}

void SoftTimerExt::startW(unsigned long delay_ms)
{
  SoftTimer::startW(delay_ms);
  _length_ms = delay_ms;
}

void SoftTimerExt::startN(uint8_t delay_ms)
{
  SoftTimer::startN(delay_ms);
  _length_ms = delay_ms;
}

/******************************************************************************/
