/*******************************************************************************
 * SoftTimer - Arduino library for simple timers.
 *
 * Provide a simple, quick to use timer object.
 * No hardware timer or interrupt is used.
 * Less code to type, and shorter programs because of not having to pepper
 * comparisons with millis() everywhere.
 *
 * Developed with Arduino IDE 1.6.10. May not run on Arduino 0.X.
 *
 * TODO:
 * - Doxygen documentation. More comments.
 *
 * Copyright (C) 2016 by Volker Kuhlmann
 *               http://volker.top.geek.nz/contact.html
 * All rights reserved.
 * Released under the terms of the GNU General Public License (GPL) Version 2.
 * See http://www.gnu.org/ for details.
 *
 * v1.0, 24 Aug 2016  Volker Kuhlmann
 *   Initial version. Based on code in a project from 19 Aug 2016.
 * v1.1, 25 Aug 2016  Volker Kuhlmann
 *   Add missing intervalS(). Change unsigned short to explicit uint8_t.
 *   Add example.
 * v1.2, 05 Sep 2016  Volker Kuhlmann
 *   Rename from SimpleTimer to SoftTimer.
 *   Rename startS/intervalS to startN/intervalN, for Narrow / Wide.
 *   Add SoftTimerExt.
 *******************************************************************************
 */

#ifndef SOFTTIMER_H_
#define SOFTTIMER_H_


class SoftTimer
{
  public:
    bool is_expired();

    void start(uint16_t delay_ms);
    void startW(unsigned long delay_ms);
    void startN(uint8_t delay_ms);

    bool interval(uint16_t delay_ms);
    bool intervalW(unsigned long delay_ms);
    bool intervalN(uint8_t delay_ms);

  protected:
    unsigned long _finish_ms;
};


class SoftTimerExt
  : public SoftTimer
{
  public:
    SoftTimerExt();
    // This ctor works like startW(), but is it really useful?
    SoftTimerExt(unsigned long delay_ms);

    void restart();

    uint32_t remaining();
    uint32_t remaining_s();
    uint32_t elapsed();
    uint32_t elapsed_s();

    bool is_new_s();

    // Re-implement:
    void start(uint16_t delay_ms);
    void startW(unsigned long delay_ms);
    void startN(uint8_t delay_ms);

  protected:
    unsigned long _length_ms;
    uint8_t _last_s;
};


#endif // SOFTTIMER_H_

/******************************************************************************/
